import logging
_logger = logging.getLogger('listentry')
_logger.setLevel(logging.DEBUG)

import gobject
import hippo
import pango
from sugar.graphics import style
from sugar.graphics.roundbox import CanvasRoundBox
from sugar import profile

from osicons import ListenIcon
from storymanager import StoryManager

class StoryEntry(hippo.CanvasBox):
  __gtype_name__ = 'StoryEntry'

  _DATE_COL_WIDTH     = style.GRID_CELL_SIZE * 3
#  _PROGRESS_COL_WIDTH = style.GRID_CELL_SIZE * 5

  def __init__(self, storyobject):
    hippo.CanvasBox.__init__(self, spacing=style.DEFAULT_SPACING,
                             padding_top=style.DEFAULT_PADDING,
                             padding_bottom=style.DEFAULT_PADDING,
                             padding_left=style.DEFAULT_PADDING * 2,
                             padding_right=style.DEFAULT_PADDING * 2,
                             box_height=style.GRID_CELL_SIZE,
                             orientation=hippo.ORIENTATION_HORIZONTAL)

    self._story_manager_handle = StoryManager()
    self._story_manager_handle.connect('stop_listen',
                                       self._stop_listen_cb)

    self._storyobj = None

    self._frame = CanvasRoundBox()
    self.append(self._frame, hippo.PACK_EXPAND)

    self._listen_icon = self._create_listen_icon()
    self._frame.append(self._listen_icon)

    self._title = self._create_title(storyobject.get_storyname())
    self._frame.append(self._title, hippo.PACK_EXPAND)

    self._date = self._create_date(storyobject.get_datestring())
    self._frame.append(self._date)

    self._frame.props.background_color = style.COLOR_TEXT_FIELD_GREY.get_int()
    self._frame.props.border = 0
    self.set_storyobj(storyobject)


  def _create_listen_icon(self):
    listen_icon =  ListenIcon()
    listen_icon.connect('button-release-event',
                        self._listen_icon_button_release_event_cb)
    return listen_icon

  def _listen_icon_button_release_event_cb(self, button, event):

    self._listen_icon.props.listening = not self._listen_icon.props.listening

    if self._listen_icon.props.listening:
      self._story_manager_handle.startListen(self._storyobj)
      self._storyobj.set_listening(True)
    else:
      self._story_manager_handle.stopListen()
      self._storyobj.set_listening(False)

    return True

  def _stop_listen_cb(self, button, event):
    if event == self._storyobj.get_oggfile():
      self._listen_icon.props.listening = False
      self._storyobj.set_listening(False)

  def _create_title(self, title):
    title = hippo.CanvasText(text=title,
                             xalign=hippo.ALIGNMENT_START,
                             font_desc=style.FONT_BOLD.get_pango_desc(),
                             size_mode=hippo.CANVAS_SIZE_WRAP_WORD)
    return title

  def _create_date(self, datestring):
    date = hippo.CanvasText(text=datestring,
                            xalign=hippo.ALIGNMENT_START,
                            font_desc=style.FONT_NORMAL.get_pango_desc(),
                            color=style.COLOR_TOOLBAR_GREY.get_int(),
                            box_width=self._DATE_COL_WIDTH)
    return date

   
  def get_storyobj(self):
    return self._storyobj

  def set_storyobj(self, storyobject):
    self._storyobj = storyobject
    self._title.props.text = self._storyobj.get_storyname()
    self._date.props.text = self._storyobj.get_datestring()
    self._listen_icon.props.listening = self._storyobj.is_listening()

  storyobj = property(get_storyobj, set_storyobj)

