import logging
_logger = logging.getLogger('listview')
_logger.setLevel(logging.DEBUG)

import os

import hippo
import gobject
import gtk

from sugar.activity import activity
from sugar.graphics import style

from listentry import StoryEntry
import query

class ListView(gtk.HBox):
    __gtype_name__ = 'OurStoriesListView'

    def __init__(self):
        self._result_set = None
        self._entries = []
        self._page_size = 0
        self._last_value = -1
        self._reflow_sid = 0

        gtk.HBox.__init__(self)
        self.set_flags(gtk.HAS_FOCUS|gtk.CAN_FOCUS)

        self._box = hippo.CanvasBox(
                        orientation=hippo.ORIENTATION_VERTICAL,
                        background_color=style.COLOR_WHITE.get_int())

        canvas = hippo.Canvas()
        canvas.set_root(self._box)

        self.pack_start(canvas)
        canvas.show()

        self._vadjustment = gtk.Adjustment(value=0, lower=0, upper=0, 
                                           step_incr=1, page_incr=0, page_size=0)
        self._vadjustment.connect('value-changed', self._vadjustment_value_changed_cb)
        self._vadjustment.connect('changed', self._vadjustment_changed_cb)

        self._vscrollbar = gtk.VScrollbar(self._vadjustment)
        self.pack_end(self._vscrollbar, expand=False, fill=False)
        self._vscrollbar.show()
        

    def _vadjustment_changed_cb(self, vadjustment):
        """
        _logger.debug('_vadjustment_changed_cb:\n \t%r\n \t%r\n \t%r\n \t%r\n \t%r\n' % \
                      (vadjustment.props.lower, vadjustment.props.page_increment, 
                      vadjustment.props.page_size, vadjustment.props.step_increment,
                      vadjustment.props.upper))
        """
        if vadjustment.props.upper > self._page_size:
            self._vscrollbar.show()
        else:
            self._vscrollbar.hide()

    def _vadjustment_value_changed_cb(self, vadjustment):
        gobject.idle_add(self._do_scroll)

    def _do_scroll(self, force=False):
        value = int(self._vadjustment.props.value)

        if value == self._last_value and not force:
            return
        self._last_value = value

        self._result_set.seek(value)

        stories = self._result_set.get_stories()

        if self._result_set.length != self._vadjustment.props.upper:
            self._vadjustment.props.upper = self._result_set.length
            self._vadjustment.changed()

        self._refresh_view(stories)
        
#        _logger.debug('_do_scroll %r \n' % (value))
        
        return False

    def _refresh_view(self, stories):
      # Refresh view and create the entries if they don't exist yet.
#      try:
        for i in range(0, self._page_size):
            if i < len(stories):
                if i >= len(self._entries):
                    entry = StoryEntry(stories[i])
                    self._box.append(entry)
                    self._entries.append(entry)
                else:
                    entry = self._entries[i]
                    entry.storyobj = stories[i]
                    entry.set_visible(True)
            elif i < len(self._entries):
                    entry = self._entries[i]
                    entry.set_visible(False)
#      except Exception, e:
#        _logger.info('exception in refresh view')

    def refresh(self):
        if self._result_set:
          del self._result_set
        self._result_set = query.ResultSet()
        self._result_set.connect('new_story_sig', self._result_set_modified_cb)
        self._vadjustment.props.upper = self._result_set.length
        self._vadjustment.changed()

        self._vadjustment.props.value = min(self._vadjustment.props.value,
                                            self._result_set.length - self._page_size)
        self._do_scroll(force=True)

    def do_focus(self, direction):
        if not self.is_focus():
            self.grab_focus()
            return True
        return False

    def do_size_allocate(self, allocation):
        gtk.HBox.do_size_allocate(self, allocation)
        new_page_size = int(allocation.height / style.GRID_CELL_SIZE)

#        _logger.debug("do_size_allocate: %r" % new_page_size)
        
        if new_page_size != self._page_size:
            self._page_size = new_page_size
            self._queue_reflow()

    def _queue_reflow(self):
        if not self._reflow_sid:
            self._reflow_sid = gobject.idle_add(self._reflow_idle_cb)

    def _reflow_idle_cb(self):
        self._box.clear()
        self._entries = []

        self._vadjustment.props.page_size = self._page_size
        self._vadjustment.props.page_increment = self._page_size
        self._vadjustment.changed()

        if self._result_set is None:
            self._result_set = query.ResultSet()
            self._result_set.connect('new_story_sig', self._result_set_modified_cb)

        max_value = max(0, self._result_set.length - self._page_size)
        if self._vadjustment.props.value > max_value:
            self._vadjustment.props.value = max_value
        else:
            self._do_scroll(force=True)

        self._reflow_sid = 0

    def _result_set_modified_cb(self, result_set):
        logging.debug('_result_set_modified_cb')
        self._do_scroll(force=True)

