#!/usr/bin/env python
# -*- Mode: Python -*-
# vi:si:et:sw=4:sts=4:ts=4

# Copyright 2007 Google Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0

# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import logging
_logger = logging.getLogger('os-voiceRecorder')
_logger.setLevel(logging.DEBUG)

import os
import gtk
import pygtk
pygtk.require('2.0')
import sys
import pygst
pygst.require('0.10')
import gst
import gst.interfaces
import threading
import gobject
gobject.threads_init()

class VoiceRecorder(gobject.GObject):
  """
  """

  __gtype_name__ = 'os_voicerecorder'

  __gsignals__ = {
    'listen_eos':     (gobject.SIGNAL_RUN_FIRST,
                       gobject.TYPE_NONE,
                       ([])),
    'record_expired': (gobject.SIGNAL_RUN_FIRST,
                       gobject.TYPE_NONE,
                       ([])),
    'record_ticker':  (gobject.SIGNAL_RUN_FIRST,
                       gobject.TYPE_NONE,
                       ([object]))
  }

  def __init__(self):
    """
    """
    gobject.GObject.__init__(self)

    self.pipes = []

    # recording length hard limit, in seconds, currently set to 30 min
    self.RECORD_LENGTH_LIMIT = 30*60
    self.PROGRESS_UPDATE_INTERVAL = 1000
    self.RECORD_PROGRESS_ID = None
    self.MUX_MESSAGE_ID = None


  def pipe(self):
    """
    """
    return self.pipes[ len(self.pipes)-1 ]

  def nextRecordPipe(self, filename):
    """
    """
    n = str(len(self.pipes))
    pipeline = gst.parse_launch(
                 'alsasrc ! audio/x-raw-int,rate=8000,channels=1,depth=8 ! wavenc ! wavparse! audioconvert ! vorbisenc ! oggmux name=audioWavenc_'+n+' ! filesink name=audioFilesink_'+n)
    audioFilesink = pipeline.get_by_name('audioFilesink_'+n)
    audioFilesink.set_property('location', filename)
    self.pipes.append(pipeline)

  def startVoiceRecord(self, filename):
    """
    """
    self.nextRecordPipe(filename)
    self.pipe().set_state(gst.STATE_READY)
    self.RECORD_PROGRESS_ID = gobject.timeout_add(self.PROGRESS_UPDATE_INTERVAL, self._record_progress_cb)
    self.pipe().set_state(gst.STATE_PLAYING)

  def _record_progress_cb(self):
    try:
      position, format = self.pipe().query_position(gst.FORMAT_TIME)
      if position/gst.SECOND > self.RECORD_LENGTH_LIMIT:
        self.emit('record_expired')
      else:
        self.emit('record_ticker', (position/gst.SECOND, self.RECORD_LENGTH_LIMIT))
 
    except:
      pass
    return True

  def stopVoiceRecord(self):
    """
    """
    self.pipe().set_state(gst.STATE_NULL)
    if self.RECORD_PROGRESS_ID:
      gobject.source_remove( self.RECORD_PROGRESS_ID )
      self.RECORD_PROGRESS_ID = None




  def nextPlayPipe(self, filename):
    """
    """
    n = str(len(self.pipes))
    pipeline = gst.parse_launch('filesrc name=audioFilesrc_'+n+' ! oggdemux ! vorbisdec ! audioconvert ! audioresample ! alsasink')

    audioFilesrc = pipeline.get_by_name('audioFilesrc_'+n)
    audioFilesrc.set_property('location', filename)
    self.pipes.append(pipeline)


  def startVoicePlay(self, filename):
    self.nextPlayPipe(filename)
    self.pipe().set_state(gst.STATE_READY)

    # watch for end-of-stream message
    bus = self.pipe().get_bus()
    bus.add_signal_watch()
    self.MUX_MESSAGE_ID = bus.connect('message', self._mux_message_cb)

    self.pipe().set_state(gst.STATE_PLAYING)

  def _mux_message_cb(self, bus, message):
    if message.type == gst.MESSAGE_EOS:
      self.emit('listen_eos')
      self.stopVoicePlay()

  def stopVoicePlay(self):
    if self.MUX_MESSAGE_ID:
      self.pipe().get_bus().disconnect(self.MUX_MESSAGE_ID)
      self.MUX_MESSAGE_ID = None
    self.pipe().set_state(gst.STATE_NULL)

